@@ -1,6 +1,6 @@
 package CGI::Session::Driver::postgresql;
 
-# $Id: postgresql.pm 394 2008-03-22 02:35:30Z markstos $
+# $Id: postgresql.pm 221 2005-09-06 02:55:32Z markstos $
 
 # CGI::Session::Driver::postgresql - PostgreSQL driver for CGI::Session
 #
@@ -15,20 +15,20 @@ use Carp "croak";
 use CGI::Session::Driver::DBI;
 use DBD::Pg qw(PG_BYTEA PG_TEXT);
 
-$CGI::Session::Driver::postgresql::VERSION = '4.29_1';
+$CGI::Session::Driver::postgresql::VERSION = '2.1';
 @CGI::Session::Driver::postgresql::ISA     = qw( CGI::Session::Driver::DBI );
 
 
 sub init {
     my $self = shift;
     my $ret = $self->SUPER::init(@_);
-
-    # Translate external ColumnType into internal value. See POD for details.
-    $self->{PgColumnType} ||= (defined $self->{ColumnType} and (lc $self->{ColumnType} eq 'binary'))
-        ? PG_BYTEA
-        : PG_TEXT
-        ;
-
+    if (defined $self->{ColumnType}) {
+        no warnings "numeric";
+        return $ret if $self->{ColumnType} == PG_BYTEA || $self->{ColumnType} == PG_TEXT;
+        $self->{ColumnType} = lc(substr($self->{ColumnType},0,1)) eq 'b' ? PG_BYTEA : PG_TEXT;
+    } else {
+        $self->{ColumnType} = PG_TEXT;
+    }
     return $ret;
 }
 
@@ -38,46 +38,39 @@ sub store {
     croak "store(): usage error" unless $sid && $datastr;
 
     my $dbh = $self->{Handle};
-    my $type = $self->{PgColumnType};
+    my $type = $self->{ColumnType};
 
     if ($type == PG_TEXT && $datastr =~ tr/\x00//) {
         croak "Unallowed characters used in session data. Please see CGI::Session::Driver::postgresql ".
             "for more information about null characters in text columns.";
     }
 
-    local $dbh->{RaiseError} = 1;
-    eval {
-        # There is a race condition were two clients could run this code concurrently,
-        # and both end up trying to insert. That's why we check for "duplicate" below
-        my $sth = $dbh->prepare(
-             "INSERT INTO " . $self->table_name . " ($self->{DataColName},$self->{IdColName})  SELECT ?, ? 
-                WHERE NOT EXISTS (SELECT 1 FROM " . $self->table_name . " WHERE $self->{IdColName}=? LIMIT 1)");
-
-        $sth->bind_param(1,$datastr,{ pg_type => $type });
-        $sth->bind_param(2, $sid);
-        $sth->bind_param(3, $sid); # in the SELECT statement
-        my $rv = '';
-        eval { $rv = $sth->execute(); };
-        if ( $rv eq '0E0' or (defined $@ and $@ =~ m/duplicate/i) ) {
-            my $sth = $dbh->prepare("UPDATE " . $self->table_name . " SET $self->{DataColName}=? WHERE $self->{IdColName}=?");
-            $sth->bind_param(1,$datastr,{ pg_type => $type });
-            $sth->bind_param(2,$sid);
-            $sth->execute;
-        } 
-        else {
-            # Nothing. Our insert has already happened
-        }
-    };
-    if ($@) { 
-      return $self->set_error( "store(): failed with message: $@ " . $dbh->errstr );
-
-    } 
-    else {
-        return 1;
-
+    my $sth = $dbh->prepare("SELECT id FROM " . $self->table_name . " WHERE id=?");
+    unless ( defined $sth ) {
+        return $self->set_error( "store(): \$sth->prepare failed with message " . $dbh->errstr );
     }
 
+    $sth->execute( $sid ) or return $self->set_error( "store(): \$sth->execute failed with message " . $dbh->errstr );
+    if ( $sth->fetchrow_array ) {
+        __ex_and_ret($dbh,"UPDATE " . $self->table_name . " SET a_session=? WHERE id=?",$datastr,$sid, $type)
+            or return $self->set_error( "store(): serialize to db failed " . $dbh->errstr );
+    } else {
+        __ex_and_ret($dbh,"INSERT INTO " . $self->table_name . " (a_session,id) VALUES(?, ?)",$datastr, $sid, $type)
+            or return $self->set_error( "store(): serialize to db failed " . $dbh->errstr );
+    }
+    return 1;
+}
 
+sub __ex_and_ret {
+    my ($dbh,$sql,$datastr,$sid,$type) = @_;
+    eval {
+        my $sth = $dbh->prepare($sql) or return 0;
+        $sth->bind_param(1,$datastr,{ pg_type => $type }) or return 0;
+        $sth->bind_param(2,$sid) or return 0;
+        $sth->execute() or return 0;
+    };
+    return 0 if $@;
+    return 1;
 }
 
 1;
@@ -113,26 +106,6 @@ and within your code use:
 
 Please note the I<ColumnType> argument. PostgreSQL's text type has problems when trying to hold a null character. (Known as C<"\0"> in Perl, not to be confused with SQL I<NULL>). If you know there is no chance of ever having a null character in the serialized data, you can leave off the I<ColumnType> attribute. Using a I<BYTEA> column type and C<< ColumnType => 'binary' >> is recommended when using L<Storable|CGI::Session::Serialize::storable> as the serializer or if there's any possibility that a null value will appear in any of the serialized data.
 
-To use different column names, change the 'create table' statement, and then simply do this:
-
-    $s = new CGI::Session('driver:pg', undef,
-    {
-        TableName=>'session',
-        IdColName=>'my_id',
-        DataColName=>'my_data',
-        DataSource=>'dbi:pg:dbname=project',
-    });
-
-or
-
-    $s = new CGI::Session('driver:pg', undef,
-    {
-        TableName=>'session',
-        IdColName=>'my_id',
-        DataColName=>'my_data',
-        Handle=>$dbh,
-    });
-
 For more details see L<CGI::Session::Driver::DBI|CGI::Session::Driver::DBI>, parent class.
 
 Also see L<sqlite driver|CGI::Session::Driver::sqlite>, which exercises different method for dealing with binary data.
