@@ -1,9 +1,9 @@
 package CGI::Session::ErrorHandler;
 
-# $Id: ErrorHandler.pm 394 2008-03-22 02:35:30Z markstos $
+# $Id: ErrorHandler.pm 216 2005-09-01 10:52:26Z sherzodr $
 
 use strict;
-$CGI::Session::ErrorHandler::VERSION = '4.29_1';
+$CGI::Session::ErrorHandler::VERSION = "4.01";
 
 =pod
 
@@ -38,11 +38,10 @@ Implicitly defines $pkg_name::errstr and sets its value to $message. Return valu
 =cut
 
 sub set_error {
-    my $class   = shift;
-    my $message = shift;
+    my $class = shift;
     $class = ref($class) || $class;
     no strict 'refs';
-    ${ "$class\::errstr" } = sprintf($message || "", @_);
+    ${ "$class\::errstr" } = $_[0] || "";
     return;
 }
 
